@@ -22,7 +22,7 @@ class Comparator(Syncer):
 
         q = "select count(1) from only _TABLE_"
         q = self.cf.get('compare_sql', q)
-        q = q.replace('_TABLE_', tbl)
+        q = q.replace('_TABLE_', skytools.quote_fqident(tbl))
 
         self.log.debug("srcdb: " + q)
         src_curs.execute(q)
