@@ -429,14 +429,13 @@ class SerialConsumer(Consumer):
             # on track
             return False
 
-        if prev_tick < dst_tick:
-            if dst_tick - prev_tick > 5:
-                 raise Exception('Difference too big, skipping dangerous')
-            self.log.warning('Got tick %d, dst has %d - skipping' % (prev_tick, dst_tick))
+        if cur_tick == dst_tick:
+            # current batch is already applied, skip it
             return True
-        else:
-            self.log.error('Got tick %d, dst has %d - ticks lost' % (prev_tick, dst_tick))
-            raise Exception('Lost ticks')
+
+        # anything else means problems
+        raise Exception('Lost position: batch %d..%d, dst has %d' % (
+                        prev_tick, cur_tick, dst_tick))
 
     def set_batch_done(self, dst_curs):
         """Helper function to set last successful batch
