@@ -195,7 +195,7 @@ def magic_insert(curs, tablename, data, fields = None, use_insert = 0):
         return
 
     # decide how to process
-    if type(data[0]) == type({}):
+    if hasattr(data[0], 'keys'):
         if fields == None:
             fields = data[0].keys()
         if use_insert:
@@ -302,7 +302,7 @@ def full_copy(tablename, src_curs, dst_curs, column_list = []):
         hdr = qtable
     if hasattr(src_curs, 'copy_expert'):
         sql_to = "COPY %s TO stdout" % hdr
-        sql_from = "COPY %s FROM stdout" % hdr
+        sql_from = "COPY %s FROM stdin" % hdr
         buf = CopyPipe(dst_curs, sql_from = sql_from)
         src_curs.copy_expert(sql_to, buf)
     else:
