@@ -389,7 +389,7 @@ class WalMgr(skytools.DBScript):
         self.log.debug("Execute rsync cmd: '%s'" % (cmd))
         if self.not_really:
             return 0
-        res = os.spawnvp(os.P_WAIT, cmdline[0], cmdline)
+        res = os.spawnvp(os.P_WAIT, cmdline[0], cmdline)       
         if res == 24:
             self.log.info("Some files vanished, but thats OK")
             res = 0
@@ -444,15 +444,20 @@ class WalMgr(skytools.DBScript):
         self.exec_big_rsync(cmdline + [ source_dir, dst_loc ])
 
 
-    def exec_cmd(self, cmdline):
+    def exec_cmd(self, cmdline,allow_error=False):
         cmd = "' '".join(cmdline)
         self.log.debug("Execute cmd: '%s'" % (cmd))
         if self.not_really:
             return
-        res = os.spawnvp(os.P_WAIT, cmdline[0], cmdline)
-        if res != 0:
+        #res = os.spawnvp(os.P_WAIT, cmdline[0], cmdline)
+        process = subprocess.Popen(cmdline,stdout=subprocess.PIPE)
+        output=process.communicate()
+        res = process.returncode
+        
+        if res != 0 and not allow_error:
             self.log.fatal("exec failed, res=%d (%s)" % (res, repr(cmdline)))
             sys.exit(1)
+        return (res,output[0])
 
     def exec_system(self, cmdline):
         self.log.debug("Execute cmd: '%s'" % (cmdline))
@@ -561,26 +566,24 @@ class WalMgr(skytools.DBScript):
 
         else:
             # disable archiving
+            cf_params = dict()
 
-            cf_params = {
-                "archive_mode": "off",
-                "wal_level": "minimal",
-                "archive_command": ""
-            }
+            if can_restart:
+                # can restart, disable archive mode and set wal_level to minimal
 
-            if curr_archive_mode is None:
-                self.log.info("archive_mode not set")
-                del(cf_params['archive_mode'])
+                cf_params['archive_command'] = ''
 
-            if curr_wal_level is None:
-                self.log.info("wal_level not set")
-                del(cf_params['wal_level'])
-
-            if not can_restart:
+                if curr_archive_mode:
+                    cf_params['archive_mode'] = 'off'
+                if curr_wal_level:
+                    cf_params['wal_level'] = 'minimal'
+                    cf_params['max_wal_senders'] = '0'
+            else:
                 # not possible to change archive_mode or wal_level (requires restart),
                 # so we just set the archive_command to /bin/true to avoid WAL pileup.
                 self.log.warning("database must be restarted to disable archiving")
                 self.log.info("Setting archive_command to /bin/true to avoid WAL pileup")
+
                 cf_params['archive_command'] = '/bin/true'
 
         self.log.debug("modifying configuration: %s" % cf_params)
@@ -623,7 +626,7 @@ class WalMgr(skytools.DBScript):
             raise Exception("invalid value for 'slave' in %s" % self.cfgfile)
         return host
 
-    def remote_walmgr(self, command, stdin_disabled = True):
+    def remote_walmgr(self, command, stdin_disabled = True,allow_error=False):
         """Pass a command to slave WalManager"""
 
         sshopt = "-T"
@@ -645,7 +648,7 @@ class WalMgr(skytools.DBScript):
         if self.not_really:
             self.log.info("remote_walmgr: %s" % command)
         else:
-            self.exec_cmd(cmdline)
+            return self.exec_cmd(cmdline,allow_error)
 
     def walmgr_setup(self):
         if self.wtype == MASTER:
@@ -729,7 +732,7 @@ class WalMgr(skytools.DBScript):
         5. Release backup lock
         """
 
-        self.remote_walmgr("xlock")
+        self.remote_xlock()
         errors = False
 
         try:
@@ -805,11 +808,11 @@ class WalMgr(skytools.DBScript):
         except Exception, e:
             self.log.error(e)
             errors = True
-
-        try:
-            self.pg_stop_backup()
-        except:
-            pass
+        finally:
+            try:
+                self.pg_stop_backup()
+            except:
+                pass
 
         try:
             self.remote_walmgr("xrelease")
@@ -1207,6 +1210,13 @@ STOP TIME: %(stop_time)s
         srcfile = os.path.join(srcdir, srcname)
         partfile = os.path.join(partdir, srcname)
 
+        # if we are using streaming replication, exit immediately 
+        # if the srcfile is not here yet
+        primary_conninfo = self.cf.get("primary_conninfo", "")
+        if primary_conninfo and not os.path.isfile(srcfile):
+            self.log.info("%s: not found (ignored)" % srcname)
+            sys.exit(1)
+  
         # assume that postgres has processed the WAL file and is 
         # asking for next - hence work not in progress anymore
         if os.path.isfile(prgrfile):
@@ -1446,13 +1456,14 @@ STOP TIME: %(stop_time)s
                 self.log.debug('using pg_controldata to determine restart points')
             restore_command = 'xrestore %f "%p"'
 
-        conf = """
-restore_command = '%s %s %s'
-#recovery_target_time=
-#recovery_target_xid=
-#recovery_target_inclusive=true
-#recovery_target_timeline=
-""" % (self.script, cf_file, restore_command)
+        conf = "restore_command = '%s %s %s'\n" % (self.script, cf_file, restore_command)
+
+        # do we have streaming replication (hot standby)
+        primary_conninfo = self.cf.get("primary_conninfo", "")
+        if primary_conninfo:
+            conf += "standby_mode = 'on'\n"
+            conf += "trigger_file = '%s'\n" % os.path.join(self.cf.get("completed_wals"), "STOP")
+            conf += "primary_conninfo = '%s'\n" % primary_conninfo
 
         self.log.info("Write %s" % rconf)
         if self.not_really:
@@ -1464,8 +1475,7 @@ restore_command = '%s %s %s'
 
         # remove stopfile on slave
         if self.wtype == SLAVE:
-            srcdir = self.cf.get("completed_wals")
-            stopfile = os.path.join(srcdir, "STOP")
+            stopfile = os.path.join(self.cf.get("completed_wals"), "STOP")
             if os.path.isfile(stopfile):
                 self.log.info("Removing stopfile: "+stopfile)
                 if not self.not_really:
@@ -1577,10 +1587,17 @@ restore_command = '%s %s %s'
         lockfile = os.path.join(srcdir, "BACKUPLOCK")
         if os.path.isfile(lockfile):
             self.log.warning("Somebody already has the backup lock.")
+            lockfilehandle = open(lockfile,"r")
+            pidstring = lockfilehandle.read();
+            try:
+                pid = int(pidstring)
+                print("%d",pid)
+            except ValueError:
+                self.log.error("lock file does not contain a pid:" + pidstring)
             return 1
 
         if not self.not_really:
-            open(lockfile, "w").write("1")
+            open(lockfile, "w").write(self.args[0])
         self.log.info("Backup lock obtained.")
         return 0
 
@@ -1773,6 +1790,22 @@ restore_command = '%s %s %s'
                     os.remove(full)
             cur_last = fname
         return cur_last
+    def remote_xlock(self):
+        ret = self.remote_walmgr("xlock " + str(os.getpid()),allow_error=True)
+        if ret[0] != 0:
+            # lock failed.
+            try:
+                lock_pid = int(ret[1])
+                if os.kill(lock_pid,0):
+                    #process exists.
+                    self.log.error("lock already obtained")
+                else:
+                    self.remote_walmgr("xrelease")
+                    ret = self.remote_walmgr("xlock " + pid(),allow_error=True)
+                    if ret[0] != 0:
+                        self.log.error("unable to obtain lock")
+            except ValueError:
+                self.log.error("error obtaining lock")
 
 if __name__ == "__main__":
     script = WalMgr(sys.argv[1:])
