@@ -5,6 +5,7 @@ create table data1 (
     id serial primary key,
     data text
 );
+cluster data1 using data1_pkey;
 
 create or replace function test_triga() returns trigger
 as $$ begin return new; end; $$ language plpgsql;
@@ -25,7 +26,7 @@ create table data2 (
 );
 
 create index idx_data2_rand on data2 (id, data);
-
+cluster data2 using idx_data2_rand;
 
 create sequence test_seq;
 select setval('test_seq', 50);
@@ -54,5 +55,34 @@ create table "Table" (
 create index "idx Table" on "Table" ("table", "I D");
 
 
+create table inh_parent1 (
+    id serial primary key,
+    data text,
+    ref1 integer,
+    constraint p1_uq_data unique (data)
+);
+
+create table inh_parent2 (
+    id serial primary key,
+    data text,
+    ref1 integer,
+    constraint p2_uq_data unique (data)
+);
+
+create table inh_mid (
+    id serial primary key,
+    data text,
+    ref1 integer,
+    constraint m_uq_data unique (data)
+) inherits (inh_parent1, inh_parent2);
+
+create table inh_child (
+    id serial primary key,
+    data text,
+    ref1 integer,
+    constraint c_uq_data unique (data)
+) inherits (inh_mid);
+
+
 
 
